'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setUrl = exports.setImage = exports.setLink = exports.removeQuote = exports.setQuote = exports.removeCode = exports.setCode = exports.removeItalic = exports.setItalic = exports.removeBold = exports.setBold = exports.removeUl = exports.setUl = exports.removeOl = exports.setOl = exports.removeH3 = exports.setH3 = exports.removeH2 = exports.setH2 = exports.removeH1 = exports.setH1 = exports.isUrl = exports.handleHeading = exports.updateUrl = exports.getUrlStyleIfActive = exports.getStyleIfActive = exports.isActiveToken = exports.getUrl = exports.getCurrentFormat = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _colors = require('material-ui/styles/colors');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var incrementPosition = function incrementPosition(summand, position) {
  var pos = Object.assign({}, position);
  pos.ch += summand;
  return pos;
};

var position = function position(line, ch) {
  return {
    line: line, ch: ch
  };
};

var isEmpty = function isEmpty(string) {
  return string.length === 0;
};

var isEmptyOneLineSelection = function isEmptyOneLineSelection(line, length) {
  return isEmpty(line) && length === 1;
};

var getPositions = function getPositions(seekedPosition, positions) {
  return positions.filter(function (pos) {
    return seekedPosition > pos[0] && seekedPosition < pos[1];
  })[0];
};

var getPlaceholderBySignature = function getPlaceholderBySignature(signature) {
  return {
    '**': 'Strong text',
    '*': 'Emphasized text',
    '- ': 'List item',
    '#. ': 'List item',
    '# ': 'Heading',
    '## ': 'Heading',
    '### ': 'Heading',
    '`': 'Code',
    '> ': 'Quote'
  }[signature];
};

var normalize = function normalize(array, signatureLength) {
  var accum = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  if (array.length) {
    /* Increases end postion with signature length */
    var chunk = array.splice(0, 2);
    chunk[1] += signatureLength;
    accum.push(chunk);
    normalize(array, signatureLength, accum);
  }
  return accum;
};

var getMatches = function getMatches(signature, string) {
  var start = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var accum = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];

  var index = string.indexOf(signature, start);
  if (index > -1) {
    accum.push(index);
    return getMatches(signature, string, index + signature.length, accum);
  }

  return normalize(accum, signature.length);
};

var getRemovingPartLength = function getRemovingPartLength(line, signature) {
  return ['- ', '### ', '## ', '# ', '> '].includes(signature) ? signature.length : line.indexOf('.') + 2;
};

var formatLink = function formatLink(pattern) {
  return function (cm) {
    return function (link) {
      var codeMirror = cm.codeMirror;

      var text = pattern.replace('##', codeMirror.getSelection()).replace('$$', link);
      codeMirror.replaceSelection(text);
    };
  };
};

var formatMultiline = function formatMultiline(signature) {
  return function (cm) {
    return function () {
      var codeMirror = cm.codeMirror;

      var start = codeMirror.getCursor('start');
      var end = codeMirror.getCursor('end');
      var length = end.line - start.line + 1;

      Array(length).fill(start.line).forEach(function (from, i) {
        var curentLine = i + 1;
        var lineNumber = from + i;
        var line = codeMirror.getLine(lineNumber);
        var text = signature + (isEmptyOneLineSelection(line, length) ? getPlaceholderBySignature(signature) : line);
        if (signature === '#. ') {
          text = text.replace(signature, curentLine + '. ');
        }
        codeMirror.replaceRange(text, position(lineNumber, 0), position(lineNumber, line.length));
      });

      var lastLineLength = codeMirror.getLine(end.line).length;
      codeMirror.setSelection(position(start.line, 0), position(end.line, lastLineLength));
      codeMirror.focus();
    };
  };
};

var removeMultiline = function removeMultiline(signature) {
  return function (cm) {
    return function () {
      var codeMirror = cm.codeMirror;

      var start = codeMirror.getCursor('start');
      var end = codeMirror.getCursor('end');
      var length = end.line - start.line + 1;

      Array(length).fill(start.line).forEach(function (from, i) {
        var currentLine = from + i;
        var line = codeMirror.getLine(currentLine);
        codeMirror.replaceRange('', position(currentLine, 0), position(currentLine, getRemovingPartLength(line, signature)));
      });
    };
  };
};

var formatInline = function formatInline(signature) {
  return function (cm) {
    return function () {
      var codeMirror = cm.codeMirror;


      var selection = codeMirror.getSelection();
      var text = isEmpty(selection) ? getPlaceholderBySignature(signature) : selection;
      var start = codeMirror.getCursor('start');
      var end = incrementPosition(text.length, start);

      codeMirror.replaceSelection(signature + text + signature);
      codeMirror.setSelection.apply(codeMirror, _toConsumableArray([start, end].map(function (pos) {
        return incrementPosition(signature.length, pos);
      })));

      codeMirror.focus();
    };
  };
};

var removeInline = function removeInline(signature) {
  return function (cm) {
    return function () {
      var codeMirror = cm.codeMirror;


      var cursor = codeMirror.getCursor('start');
      var line = codeMirror.getLine(cursor.line);

      var _getPositions = getPositions(cursor.ch, getMatches(signature, line)),
          _getPositions2 = _slicedToArray(_getPositions, 2),
          start = _getPositions2[0],
          end = _getPositions2[1];

      var startPoint = position(cursor.line, start);
      var endPoint = position(cursor.line, end);
      var text = codeMirror.getRange(startPoint, endPoint).split(signature).join('');

      codeMirror.replaceRange(text, startPoint, endPoint);
    };
  };
};

var normalizeList = function normalizeList(types, line) {
  var typesCloned = [].concat(_toConsumableArray(types));
  if (typesCloned[0] !== 'variable-2') return typesCloned;
  typesCloned[0] = line.startsWith('- ') || line.startsWith('* ') ? typesCloned[0] = 'ul' : typesCloned[0] = 'ol';

  return typesCloned;
};

var getCurrentFormat = exports.getCurrentFormat = function getCurrentFormat(cm) {
  var codeMirror = cm.codeMirror;

  var cursor = codeMirror.getCursor('start');
  var type = codeMirror.getTokenTypeAt(cursor);
  var line = codeMirror.getLine(cursor.line);

  return type ? normalizeList(type.split(' '), line) : [];
};

var findUrlSiblingPosition = function findUrlSiblingPosition(line, pos) {
  return line[pos - 1] === ']' || pos === 0 ? pos - 1 : findUrlSiblingPosition(line, pos - 1);
};

var getUrl = exports.getUrl = function getUrl(cm) {
  var token = cm.getTokenAt(cm.getCursor());
  return token.string;
};

var isActiveToken = exports.isActiveToken = function isActiveToken(token, tokens) {
  var index = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  return tokens.length && tokens[index] === token;
};

var getStyleIfActive = exports.getStyleIfActive = function getStyleIfActive(tokens) {
  return function (token) {
    return isActiveToken(token, tokens) ? { backgroundColor: _colors.grey400 } : {};
  };
};

var getUrlStyleIfActive = exports.getUrlStyleIfActive = function getUrlStyleIfActive(cm) {
  return function (token) {
    var codeMirror = cm.codeMirror;

    var _codeMirror$getCursor = codeMirror.getCursor(),
        line = _codeMirror$getCursor.line,
        ch = _codeMirror$getCursor.ch;

    var siblingPos = findUrlSiblingPosition(codeMirror.getLine(line), ch);
    var tokens = codeMirror.getTokenTypeAt({ line: line, ch: siblingPos }) || '';
    return getStyleIfActive(tokens.split(' '))(token);
  };
};

var updateUrl = exports.updateUrl = function updateUrl(cm, url) {
  var codeMirror = cm.codeMirror;

  var cursor = codeMirror.getCursor();
  var token = codeMirror.getTokenAt(cursor);
  codeMirror.replaceRange(url, position(cursor.line, token.start), position(cursor.line, token.end));
};

var handleHeading = exports.handleHeading = function handleHeading(schema) {
  return function (e, object) {
    schema[parseInt(object.key, 10)]();
  };
};

var isUrl = exports.isUrl = function isUrl(tokens) {
  return tokens[1] && tokens[1] === 'url';
};

var setH1 = exports.setH1 = formatMultiline('# ');

var removeH1 = exports.removeH1 = removeMultiline('# ');

var setH2 = exports.setH2 = formatMultiline('## ');

var removeH2 = exports.removeH2 = removeMultiline('## ');

var setH3 = exports.setH3 = formatMultiline('### ');

var removeH3 = exports.removeH3 = removeMultiline('### ');

var setOl = exports.setOl = formatMultiline('#. ');

var removeOl = exports.removeOl = removeMultiline('#. ');

var setUl = exports.setUl = formatMultiline('- ');

var removeUl = exports.removeUl = removeMultiline('- ');

var setBold = exports.setBold = formatInline('**');

var removeBold = exports.removeBold = removeInline('**');

var setItalic = exports.setItalic = formatInline('*');

var removeItalic = exports.removeItalic = removeInline('*');

var setCode = exports.setCode = formatInline('`');

var removeCode = exports.removeCode = removeInline('`');

var setQuote = exports.setQuote = formatMultiline('> ');

var removeQuote = exports.removeQuote = removeMultiline('> ');

var setLink = exports.setLink = formatLink('[##]($$)');

var setImage = exports.setImage = formatLink('![##]($$)');

var setUrl = exports.setUrl = function setUrl(cm, url, isImageDialog) {
  return isImageDialog ? setImage(cm)(url) : setLink(cm)(url);
};